@@ -39,6 +39,8 @@ module Ganeti.DataCollectors.CLI
   , oNode
   , oConfdAddr
   , oConfdPort
+  , oInputFile
+  , oInstances
   , genericOptions
   ) where
 
@@ -64,6 +66,11 @@ data Options = Options
   , optConfdAddr   :: Maybe String   -- ^ IP address of the Confd server
   , optConfdPort   :: Maybe Int      -- ^ The port of the Confd server to
                                      -- connect to
+  , optInputFile   :: Maybe FilePath -- ^ Path to the file containing the
+                                     -- information to be parsed
+  , optInstances   :: Maybe FilePath -- ^ Path to the file contained a
+                                     -- serialized list of instances as in:
+                                     -- ([Primary], [Secondary])
   } deriving Show
 
 -- | Default values for the command line options.
@@ -77,6 +84,8 @@ defaultOptions  = Options
   , optNode        = Nothing
   , optConfdAddr   = Nothing
   , optConfdPort   = Nothing
+  , optInputFile   = Nothing
+  , optInstances   = Nothing
   }
 
 -- | Abbreviation for the option type.
@@ -127,6 +136,20 @@ oConfdPort =
     "Network port of the Confd server to connect to",
     OptComplInteger)
 
+oInputFile :: OptType
+oInputFile =
+  ( Option "f" ["file"]
+      (ReqArg (\ f o -> Ok o { optInputFile = Just f }) "FILE")
+      "the input FILE",
+    OptComplFile)
+
+oInstances :: OptType
+oInstances =
+  ( Option "i" ["instances"]
+      (ReqArg (\ f o -> Ok o { optInstances = Just f}) "FILE")
+      "the FILE containing serialized instances",
+    OptComplFile)
+
 -- | Generic options.
 genericOptions :: [GenericOptType Options]
 genericOptions =  [ oShowVer
