@@ -50,6 +50,7 @@ module Ganeti.OpParams
   , SetParamsMods(..)
   , ExportTarget(..)
   , pInstanceName
+  , pInstanceUuid
   , pInstances
   , pName
   , pTagsList
@@ -61,7 +62,9 @@ module Ganeti.OpParams
   , pForce
   , pIgnoreOfflineNodes
   , pNodeName
+  , pNodeUuid
   , pNodeNames
+  , pNodeUuids
   , pGroupName
   , pMigrationMode
   , pMigrationLive
@@ -82,7 +85,9 @@ module Ganeti.OpParams
   , pIpConflictsCheck
   , pNoRemember
   , pMigrationTargetNode
+  , pMigrationTargetNodeUuid
   , pMoveTargetNode
+  , pMoveTargetNodeUuid
   , pStartupPaused
   , pVerbose
   , pDebugSimulateErrors
@@ -100,6 +105,8 @@ module Ganeti.OpParams
   , pOptDiskTemplate
   , pFileDriver
   , pFileStorageDir
+  , pGlobalFileStorageDir
+  , pGlobalSharedFileStorageDir
   , pVgName
   , pEnabledHypervisors
   , pHypervisor
@@ -145,6 +152,7 @@ module Ganeti.OpParams
   , pNames
   , pNodes
   , pRequiredNodes
+  , pRequiredNodeUuids
   , pStorageType
   , pStorageChanges
   , pMasterCandidate
@@ -154,17 +162,21 @@ module Ganeti.OpParams
   , pPowered
   , pIallocator
   , pRemoteNode
+  , pRemoteNodeUuid
   , pEvacMode
   , pInstCreateMode
   , pNoInstall
   , pInstOs
   , pPrimaryNode
+  , pPrimaryNodeUuid
   , pSecondaryNode
+  , pSecondaryNodeUuid
   , pSourceHandshake
   , pSourceInstance
   , pSourceShutdownTimeout
   , pSourceX509Ca
   , pSrcNode
+  , pSrcNodeUuid
   , pSrcPath
   , pStartInstance
   , pInstTags
@@ -189,6 +201,7 @@ module Ganeti.OpParams
   , pTargetGroups
   , pExportMode
   , pExportTargetNode
+  , pExportTargetNodeUuid
   , pRemoveInstance
   , pIgnoreRemoveFailures
   , pX509KeyName
@@ -201,6 +214,7 @@ module Ganeti.OpParams
   , pDelayDuration
   , pDelayOnMaster
   , pDelayOnNodes
+  , pDelayOnNodeUuids
   , pDelayRepeat
   , pIAllocatorDirection
   , pIAllocatorMode
@@ -512,6 +526,10 @@ instance JSON ExportTarget where
 pInstanceName :: Field
 pInstanceName = simpleField "instance_name" [t| String |]
 
+-- | An instance UUID (for single-instance LUs).
+pInstanceUuid :: Field
+pInstanceUuid = optionalField $ simpleField "instance_uuid" [t| String |]
+
 -- | A list of instances.
 pInstances :: Field
 pInstances = defaultField [| [] |] $
@@ -564,11 +582,20 @@ pIgnoreOfflineNodes = defaultFalse "ignore_offline_nodes"
 pNodeName :: Field
 pNodeName = simpleField "node_name" [t| NonEmptyString |]
 
+-- | A node UUID (for single-node LUs).
+pNodeUuid :: Field
+pNodeUuid = optionalField $ simpleField "node_uuid" [t| NonEmptyString |]
+
 -- | List of nodes.
 pNodeNames :: Field
 pNodeNames =
   defaultField [| [] |] $ simpleField "node_names" [t| [NonEmptyString] |]
 
+-- | List of node UUIDs.
+pNodeUuids :: Field
+pNodeUuids =
+  optionalField $ simpleField "node_uuids" [t| [NonEmptyString] |]
+
 -- | A required node group name (for single-group LUs).
 pGroupName :: Field
 pGroupName = simpleField "group_name" [t| NonEmptyString |]
@@ -657,12 +684,22 @@ pNoRemember = defaultFalse "no_remember"
 pMigrationTargetNode :: Field
 pMigrationTargetNode = optionalNEStringField "target_node"
 
+-- | Target node UUID for instance migration/failover.
+pMigrationTargetNodeUuid :: Field
+pMigrationTargetNodeUuid = optionalNEStringField "target_node_uuid"
+
 -- | Target node for instance move (required).
 pMoveTargetNode :: Field
 pMoveTargetNode =
   renameField "MoveTargetNode" $
   simpleField "target_node" [t| NonEmptyString |]
 
+-- | Target node UUID for instance move.
+pMoveTargetNodeUuid :: Field
+pMoveTargetNodeUuid =
+  renameField "MoveTargetNodeUuid" . optionalField $
+  simpleField "target_node_uuid" [t| NonEmptyString |]
+
 -- | Pause instance at startup.
 pStartupPaused :: Field
 pStartupPaused = defaultFalse "startup_paused"
@@ -752,6 +789,14 @@ pFileDriver = optionalField $ simpleField "file_driver" [t| FileDriver |]
 pFileStorageDir :: Field
 pFileStorageDir = optionalNEStringField "file_storage_dir"
 
+-- | Global directory for storing file-backed disks.
+pGlobalFileStorageDir :: Field
+pGlobalFileStorageDir = optionalNEStringField "file_storage_dir"
+
+-- | Global directory for storing shared-file-backed disks.
+pGlobalSharedFileStorageDir :: Field
+pGlobalSharedFileStorageDir = optionalNEStringField "shared_file_storage_dir"
+
 -- | Volume group name.
 pVgName :: Field
 pVgName = optionalStringField "vg_name"
@@ -995,6 +1040,12 @@ pRequiredNodes :: Field
 pRequiredNodes =
   renameField "ReqNodes " $ simpleField "nodes" [t| [NonEmptyString] |]
 
+-- | Required list of node names.
+pRequiredNodeUuids :: Field
+pRequiredNodeUuids =
+  renameField "ReqNodeUuids " . optionalField $
+    simpleField "node_uuids" [t| [NonEmptyString] |]
+
 -- | Storage type.
 pStorageType :: Field
 pStorageType = simpleField "storage_type" [t| StorageType |]
@@ -1032,6 +1083,10 @@ pIallocator = optionalNEStringField "iallocator"
 pRemoteNode :: Field
 pRemoteNode = optionalNEStringField "remote_node"
 
+-- | New secondary node UUID.
+pRemoteNodeUuid :: Field
+pRemoteNodeUuid = optionalNEStringField "remote_node_uuid"
+
 -- | Node evacuation mode.
 pEvacMode :: Field
 pEvacMode = renameField "EvacMode" $ simpleField "mode" [t| NodeEvacMode |]
@@ -1053,10 +1108,18 @@ pInstOs = optionalNEStringField "os_type"
 pPrimaryNode :: Field
 pPrimaryNode = optionalNEStringField "pnode"
 
+-- | Primary node UUID for an instance.
+pPrimaryNodeUuid :: Field
+pPrimaryNodeUuid = optionalNEStringField "pnode_uuid"
+
 -- | Secondary node for an instance.
 pSecondaryNode :: Field
 pSecondaryNode = optionalNEStringField "snode"
 
+-- | Secondary node UUID for an instance.
+pSecondaryNodeUuid :: Field
+pSecondaryNodeUuid = optionalNEStringField "snode_uuid"
+
 -- | Signed handshake from source (remote import only).
 pSourceHandshake :: Field
 pSourceHandshake =
@@ -1081,6 +1144,10 @@ pSourceX509Ca = optionalNEStringField "source_x509_ca"
 pSrcNode :: Field
 pSrcNode = optionalNEStringField "src_node"
 
+-- | Source node for import.
+pSrcNodeUuid :: Field
+pSrcNodeUuid = optionalNEStringField "src_node_uuid"
+
 -- | Source directory for import.
 pSrcPath :: Field
 pSrcPath = optionalNEStringField "src_path"
@@ -1186,6 +1253,12 @@ pExportTargetNode =
   renameField "ExportTarget" $
   simpleField "target_node" [t| ExportTarget |]
 
+-- | Export target node UUID field.
+pExportTargetNodeUuid :: Field
+pExportTargetNodeUuid =
+  renameField "ExportTargetNodeUuid" . optionalField $
+  simpleField "target_node_uuid" [t| NonEmptyString |]
+
 -- | Whether to remove instance after export.
 pRemoveInstance :: Field
 pRemoveInstance = defaultFalse "remove_instance"
@@ -1246,6 +1319,12 @@ pDelayOnNodes =
   defaultField [| [] |] $
   simpleField "on_nodes" [t| [NonEmptyString] |]
 
+-- | on_node_uuids field for 'OpTestDelay'.
+pDelayOnNodeUuids :: Field
+pDelayOnNodeUuids =
+  renameField "DelayOnNodeUuids" . optionalField $
+  simpleField "on_node_uuids" [t| [NonEmptyString] |]
+
 -- | Repeat parameter for OpTestDelay.
 pDelayRepeat :: Field
 pDelayRepeat =
