@@ -97,19 +97,19 @@ class TestClusterObject(unittest.TestCase):
                                           os_name="lenny-image"),
                          cl.os_hvp["lenny-image"][constants.HT_XEN_PVM])
 
-
   def testFillHvFullMerge(self):
     inst_hvparams = {
       "blah": "blubb",
       }
 
-    fake_dict = {
+    fake_dict = constants.HVC_DEFAULTS[constants.HT_FAKE].copy()
+    fake_dict.update({
       "foo": "baz",
       "bar": "foo",
       "foobar": "foobar",
       "blah": "blubb",
       "blubb": "blah",
-      }
+      })
     fake_inst = objects.Instance(name="foobar",
                                  os="lenny-image",
                                  hypervisor=constants.HT_FAKE,
@@ -223,6 +223,33 @@ class TestClusterObject(unittest.TestCase):
     cluster = objects.Cluster(ipolicy={"unknown_key": None})
     self.assertRaises(errors.ConfigurationError, cluster.UpgradeConfig)
 
+  def testUpgradeEnabledDiskTemplates(self):
+    cfg = objects.ConfigData()
+    cfg.cluster = objects.Cluster()
+    cfg.cluster.volume_group_name = "myvg"
+    instance1 = objects.Instance()
+    instance1.disk_template = constants.DT_DISKLESS
+    instance2 = objects.Instance()
+    instance2.disk_template = constants.DT_RBD
+    cfg.instances = { "myinstance1": instance1, "myinstance2": instance2 }
+    nodegroup = objects.NodeGroup()
+    nodegroup.ipolicy = {}
+    nodegroup.ipolicy[constants.IPOLICY_DTS] = [instance1.disk_template, \
+      constants.DT_BLOCK]
+    cfg.cluster.ipolicy = {}
+    cfg.cluster.ipolicy[constants.IPOLICY_DTS] = \
+      [constants.DT_EXT, constants.DT_DISKLESS]
+    cfg.nodegroups = { "mynodegroup": nodegroup }
+    cfg._UpgradeEnabledDiskTemplates()
+    expected_disk_templates = [constants.DT_DRBD8,
+                               constants.DT_PLAIN,
+                               instance1.disk_template,
+                               instance2.disk_template]
+    self.assertEqual(set(expected_disk_templates),
+                     set(cfg.cluster.enabled_disk_templates))
+    self.assertEqual(set([instance1.disk_template]),
+                     set(cfg.cluster.ipolicy[constants.IPOLICY_DTS]))
+
 
 class TestClusterObjectTcpUdpPortPool(unittest.TestCase):
   def testNewCluster(self):
@@ -298,9 +325,9 @@ class TestInstance(unittest.TestCase):
     inst = objects.Instance(name="fakeinstplain.example.com",
       primary_node="node3.example.com",
       disks=[
-        objects.Disk(dev_type=constants.LD_LV, size=128,
+        objects.Disk(dev_type=constants.DT_PLAIN, size=128,
                      logical_id=("myxenvg", "disk25494")),
-        objects.Disk(dev_type=constants.LD_LV, size=512,
+        objects.Disk(dev_type=constants.DT_PLAIN, size=512,
                      logical_id=("myxenvg", "disk29071")),
         ])
 
@@ -315,13 +342,13 @@ class TestInstance(unittest.TestCase):
     inst = objects.Instance(name="fakeinstdrbd.example.com",
       primary_node="node10.example.com",
       disks=[
-        objects.Disk(dev_type=constants.LD_DRBD8, size=786432,
+        objects.Disk(dev_type=constants.DT_DRBD8, size=786432,
           logical_id=("node10.example.com", "node15.example.com",
                       12300, 0, 0, "secret"),
           children=[
-            objects.Disk(dev_type=constants.LD_LV, size=786432,
+            objects.Disk(dev_type=constants.DT_PLAIN, size=786432,
                          logical_id=("myxenvg", "disk0")),
-            objects.Disk(dev_type=constants.LD_LV, size=128,
+            objects.Disk(dev_type=constants.DT_PLAIN, size=128,
                          logical_id=("myxenvg", "meta0"))
           ],
           iv_name="disk/0")
@@ -368,7 +395,7 @@ class TestNode(unittest.TestCase):
 
   def testDiskState(self):
     node = objects.Node(name="node32087.example.com", disk_state={
-      constants.LD_LV: {
+      constants.DT_PLAIN: {
         "lv32352": objects.NodeDiskState(total=128),
         "lv2082": objects.NodeDiskState(total=512),
         },
@@ -381,14 +408,12 @@ class TestNode(unittest.TestCase):
 
     self.assertEqual(node2.name, "node32087.example.com")
     self.assertEqual(frozenset(node2.disk_state), frozenset([
-      constants.LD_LV,
+      constants.DT_PLAIN,
       ]))
-    self.assertEqual(frozenset(node2.disk_state[constants.LD_LV]), frozenset([
-      "lv32352",
-      "lv2082",
-      ]))
-    self.assertEqual(node2.disk_state[constants.LD_LV]["lv2082"].total, 512)
-    self.assertEqual(node2.disk_state[constants.LD_LV]["lv32352"].total, 128)
+    self.assertEqual(frozenset(node2.disk_state[constants.DT_PLAIN]),
+                     frozenset(["lv32352", "lv2082"]))
+    self.assertEqual(node2.disk_state[constants.DT_PLAIN]["lv2082"].total, 512)
+    self.assertEqual(node2.disk_state[constants.DT_PLAIN]["lv32352"].total, 128)
 
   def testFilterEsNdp(self):
     node1 = objects.Node(name="node11673.example.com", ndparams={
@@ -687,5 +712,36 @@ class TestInstancePolicy(unittest.TestCase):
     self.assertTrue(INVALID_KEY in policy)
 
 
+class TestDisk(unittest.TestCase):
+  def addChild(self, disk):
+    """Adds a child of the same device type as the parent."""
+    disk.children = []
+    child = objects.Disk()
+    child.dev_type = disk.dev_type
+    disk.children.append(child)
+
+  def testUpgradeConfigDevTypeLegacy(self):
+    for old, new in [("drbd8", constants.DT_DRBD8),
+                     ("lvm", constants.DT_PLAIN)]:
+      disk = objects.Disk()
+      disk.dev_type = old
+      self.addChild(disk)
+      disk.UpgradeConfig()
+      self.assertEqual(new, disk.dev_type)
+      self.assertEqual(new, disk.children[0].dev_type)
+
+  def testUpgradeConfigDevTypeLegacyUnchanged(self):
+    dev_types = [constants.DT_FILE, constants.DT_SHARED_FILE,
+                 constants.DT_BLOCK, constants.DT_EXT,
+                 constants.DT_RBD]
+    for dev_type in dev_types:
+      disk = objects.Disk()
+      disk.dev_type = dev_type
+      self.addChild(disk)
+      disk.UpgradeConfig()
+      self.assertEqual(dev_type, disk.dev_type)
+      self.assertEqual(dev_type, disk.children[0].dev_type)
+
+
 if __name__ == "__main__":
   testutils.GanetiTestProgram()
